@@ -1,3 +1,4 @@
+use super::ScannerConfig;
 use crate::parser::{try_to_convert, ParolGrammar};
 use crate::Cfg;
 use std::convert::TryFrom;
@@ -37,27 +38,9 @@ pub struct GrammarConfig {
     pub comment: Option<String>,
 
     ///
-    /// Strings with the characters that starts line comments
+    /// At least one scanner configurations
     ///
-    pub line_comments: Vec<String>,
-
-    ///
-    /// (String, String) tuples with the characters that start and end
-    /// a block comments, respectively.
-    ///
-    pub block_comments: Vec<(String, String)>,
-
-    ///
-    /// If true the lexer handles (and skips) newlines.
-    /// If false the user has to handle newlines on its own.
-    ///
-    pub auto_newline: bool,
-
-    ///
-    /// If true the lexer handles (and skips) whitespace.
-    /// If false the user has to handle whitespace on its own.
-    ///
-    pub auto_ws: bool,
+    pub scanner_configurations: Vec<ScannerConfig>,
 
     ///
     /// The maximum lookahead size, used for lexer generation
@@ -71,10 +54,7 @@ impl GrammarConfig {
             cfg,
             title: None,
             comment: None,
-            line_comments: Vec::new(),
-            block_comments: Vec::new(),
-            auto_newline: true,
-            auto_ws: true,
+            scanner_configurations: Vec::new(),
             lookahead_size,
         }
     }
@@ -89,23 +69,8 @@ impl GrammarConfig {
         self
     }
 
-    pub fn with_line_comments(mut self, line_comments: Vec<String>) -> Self {
-        self.line_comments = line_comments;
-        self
-    }
-
-    pub fn with_block_comments(mut self, block_comments: Vec<(String, String)>) -> Self {
-        self.block_comments = block_comments;
-        self
-    }
-
-    pub fn with_auto_newline(mut self, auto_newline: bool) -> Self {
-        self.auto_newline = auto_newline;
-        self
-    }
-
-    pub fn with_auto_ws(mut self, auto_ws: bool) -> Self {
-        self.auto_ws = auto_ws;
+    pub fn add_scanner(mut self, scanner_config: ScannerConfig) -> Self {
+        self.scanner_configurations.push(scanner_config);
         self
     }
 
@@ -122,51 +87,21 @@ impl GrammarConfig {
     /// generator.
     ///
     pub fn generate_augmented_terminals(&self) -> Vec<String> {
-        let mut terminals = vec![
+        let terminals = vec![
+            "UNMATCHABLE_TOKEN".to_owned(),
+            "UNMATCHABLE_TOKEN".to_owned(),
+            "UNMATCHABLE_TOKEN".to_owned(),
+            "UNMATCHABLE_TOKEN".to_owned(),
             "UNMATCHABLE_TOKEN".to_owned(),
-            if self.auto_newline {
-                "NEW_LINE_TOKEN".to_owned()
-            } else {
-                "UNMATCHABLE_TOKEN".to_owned()
-            },
-            if self.auto_ws {
-                "WHITESPACE_TOKEN".to_owned()
-            } else {
-                "UNMATCHABLE_TOKEN".to_owned()
-            },
         ];
-        if !self.line_comments.is_empty() {
-            let line_comments_rx = self
-                .line_comments
-                .iter()
-                .map(|s| format!(r###"({}.*(\r\n|\r|\n|$))"###, s))
-                .collect::<Vec<String>>()
-                .join("|");
-            terminals.push(line_comments_rx);
-        } else {
-            terminals.push("UNMATCHABLE_TOKEN".to_owned());
-        }
-        if !self.block_comments.is_empty() {
-            let block_comments_rx = self
-                .block_comments
-                .iter()
-                .map(|(s, e)| format!(r###"((?ms){}.*?{})"###, s, e))
-                .collect::<Vec<String>>()
-                .join("|");
-            terminals.push(block_comments_rx);
-        } else {
-            terminals.push("UNMATCHABLE_TOKEN".to_owned());
-        }
-
         let mut terminals =
             self.cfg
                 .get_ordered_terminals()
                 .iter()
-                .fold(terminals, |mut acc, t| {
+                .fold(terminals, |mut acc, (t, _)| {
                     acc.push(t.to_string());
                     acc
                 });
-
         terminals.push("ERROR_TOKEN".to_owned());
         terminals
     }
@@ -178,10 +113,7 @@ impl Default for GrammarConfig {
             cfg: Cfg::default(),
             title: None,
             comment: None,
-            line_comments: Vec::new(),
-            block_comments: Vec::new(),
-            auto_newline: true,
-            auto_ws: true,
+            scanner_configurations: vec![ScannerConfig::default()], // There must always be a default scanner
             lookahead_size: 0,
         }
     }
@@ -191,10 +123,9 @@ impl Display for GrammarConfig {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::result::Result<(), Error> {
         writeln!(f, "title: {:?}", self.title)?;
         writeln!(f, "comment: {:?}", self.comment)?;
-        writeln!(f, "line_comments: {:?}", self.line_comments)?;
-        writeln!(f, "block_comments: {:?}", self.block_comments)?;
-        writeln!(f, "auto_newline: {:?}", self.auto_newline)?;
-        writeln!(f, "auto_ws: {:?}", self.auto_ws)?;
+        for sc in &self.scanner_configurations {
+            writeln!(f, "{}", sc)?;
+        }
         writeln!(f, "cfg: {:?}", self.cfg)
     }
 }
@@ -208,47 +139,54 @@ impl TryFrom<ParolGrammar> for GrammarConfig {
 
 #[cfg(test)]
 mod test {
-    use crate::generators::grammar_config::GrammarConfig;
+    use crate::generators::{GrammarConfig, ScannerConfig};
     use crate::{Cfg, Pr, Symbol};
 
     #[test]
     fn check_generate_augmented_terminals() {
         let g = Cfg::with_start_symbol("S")
-            .add_pr(Pr::new("S", vec![Symbol::t("a"), Symbol::n("X")]))
-            .add_pr(Pr::new("X", vec![Symbol::t("b"), Symbol::n("S")]))
+            .add_pr(Pr::new("S", vec![Symbol::t("a", vec![0]), Symbol::n("X")]))
+            .add_pr(Pr::new("X", vec![Symbol::t("b", vec![0]), Symbol::n("S")]))
             .add_pr(Pr::new(
                 "X",
                 vec![
-                    Symbol::t("a"),
+                    Symbol::t("a", vec![0]),
                     Symbol::n("Y"),
-                    Symbol::t("b"),
+                    Symbol::t("b", vec![0]),
                     Symbol::n("Y"),
                 ],
             ))
-            .add_pr(Pr::new("Y", vec![Symbol::t("b"), Symbol::t("a")]))
-            .add_pr(Pr::new("Y", vec![Symbol::t("a"), Symbol::n("Z")]))
+            .add_pr(Pr::new(
+                "Y",
+                vec![Symbol::t("b", vec![0]), Symbol::t("a", vec![0])],
+            ))
+            .add_pr(Pr::new("Y", vec![Symbol::t("a", vec![0]), Symbol::n("Z")]))
             .add_pr(Pr::new(
                 "Z",
-                vec![Symbol::t("a"), Symbol::n("Z"), Symbol::n("X")],
+                vec![Symbol::t("a", vec![0]), Symbol::n("Z"), Symbol::n("X")],
             ));
 
         let title = Some("Test grammar".to_owned());
         let comment = Some("A simple grammar".to_owned());
 
+        let scanner_config = ScannerConfig::default()
+            .with_line_comments(vec!["//".to_owned()])
+            .with_block_comments(vec![(r#"/\*"#.to_owned(), r#"\*/"#.to_owned())]);
+
         let grammar_config = GrammarConfig::new(g, 1)
             .with_title(title)
             .with_comment(comment)
-            .with_line_comments(vec!["//".to_owned()])
-            .with_block_comments(vec![(r#"/\*"#.to_owned(), r#"\*/"#.to_owned())]);
+            .add_scanner(scanner_config);
+
         let augment_terminals = grammar_config.generate_augmented_terminals();
 
         assert_eq!(
             vec![
                 "UNMATCHABLE_TOKEN",
-                "NEW_LINE_TOKEN",
-                "WHITESPACE_TOKEN",
-                r###"(//.*(\r\n|\r|\n|$))"###,
-                r###"((?ms)/\*.*?\*/)"###,
+                "UNMATCHABLE_TOKEN",
+                "UNMATCHABLE_TOKEN",
+                "UNMATCHABLE_TOKEN",
+                "UNMATCHABLE_TOKEN",
                 r###"a"###,
                 r###"b"###,
                 "ERROR_TOKEN"
@@ -258,5 +196,27 @@ mod test {
             .collect::<Vec<String>>(),
             augment_terminals
         );
+
+        let (special_terminals, terminal_indices, scanner_name) = grammar_config
+            .scanner_configurations[0]
+            .generate_build_information(&grammar_config.cfg);
+
+        assert_eq!(
+            vec![
+                "UNMATCHABLE_TOKEN",
+                "NEW_LINE_TOKEN",
+                "WHITESPACE_TOKEN",
+                r###"(//.*(\r\n|\r|\n|$))"###,
+                r###"((?ms)/\*.*?\*/)"###,
+            ]
+            .iter()
+            .map(|t| (*t).to_owned())
+            .collect::<Vec<String>>(),
+            special_terminals
+        );
+
+        assert_eq!(vec![5, 6], terminal_indices);
+
+        assert_eq!("INITIAL", scanner_name);
     }
 }
